require 'test/unit'

# Base class for Unit Test these required building some targets.
#
# Sample usage:
#   class TC_LibPath < Test::Unit::TestCase
#     include TestWithCompilation
#
#     test_path 'tests/lib/lib_path'
#
#     test_case :link do
#       assert_nothing_raised {
#         build 'build.rb'
#         clean 'build.rb'
#       }
#     end
#
#     test_case :unsuccesful_build do
#       assert_nothing_raised {
#         build_failure 'build.rb'
#       }
#     end
#   end
#   
module TestWithCompilation

  def init_compilation_params( path_name )
    @path_name = path_name
    @lib_path = '../' * (path_name.count( '/' ) + 1) + 'lib'
    @need_change_cwd = File.exist?( path_name )
  end
 
  def build( *project_files )
    # Running all projects to build targets.
    project_files.each do |prj|
      command = "ruby -I#{@lib_path} #{prj}"
      run_and_check_command( command )
    end
  end

  def build_failure( *project_files )
    # Running all projects to build targets.
    project_files.each do |prj|
      command = "ruby -I#{@lib_path} #{prj}"
      run_and_check_command_failure( command )
    end
  end

  def clean( *project_files )
    # Running all projects to clean up targets.
    project_files.each do |prj|
      command = "ruby -I#{@lib_path} #{prj} --mxx-clean"
      run_and_check_command( command )
    end
  end

  def run_and_check_command( command )
    started = system( command )
    assert(
      started && $?.exitstatus == 0,
      "some problems in running #{command}: " +
        "started: #{started}, exit status: #{$?}" )
  end

  def run_and_check_command_failure( command )
    started = system( command )
    assert(
      !started && $?.exitstatus != 0,
      "command '#{command}' expected to fail" )
  end

  def TestWithCompilation.included( mod )
    mod.instance_eval %Q{
      def test_path( path )
        send :define_method, :initialize do |test_case_name|
          super test_case_name
          init_compilation_params path
        end
      end

      def test_case( case_name, &block )
        send :define_method, "test_" + case_name.to_s do
          if @need_change_cwd
            Dir.chdir( @path_name ) do
              instance_eval &block
            end
          else
            instance_eval &block
          end
        end
      end
    }
  end

end

if $0 == __FILE__
  class TC_Check < Test::Unit::TestCase
    include TestWithCompilation

    test_path 'tests'

    test_case :check_attrs do
      assert_equal( 'tests', @path_name )
      assert_equal( '../lib', @lib_path )
      assert_equal( false, @need_change_cwd )
    end
  end
end

